@@ -15,23 +15,29 @@
 
 """Endpoint action implementations"""
 
+import logging
+
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 import six
 
-from openstackclient.common import command
-from openstackclient.common import utils
 from openstackclient.i18n import _
 from openstackclient.identity import common
 
 
+LOG = logging.getLogger(__name__)
+
+
 class CreateEndpoint(command.ShowOne):
-    """Create new endpoint"""
+    _description = _("Create new endpoint")
 
     def get_parser(self, prog_name):
         parser = super(CreateEndpoint, self).get_parser(prog_name)
         parser.add_argument(
             'service',
             metavar='<service>',
-            help=_('New endpoint service (name or ID)'),
+            help=_('Service to be associated with new endpoint (name or ID)'),
         )
         parser.add_argument(
             '--publicurl',
@@ -74,24 +80,40 @@ class CreateEndpoint(command.ShowOne):
 
 
 class DeleteEndpoint(command.Command):
-    """Delete endpoint"""
+    _description = _("Delete endpoint(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteEndpoint, self).get_parser(prog_name)
         parser.add_argument(
-            'endpoint',
+            'endpoints',
             metavar='<endpoint-id>',
-            help=_('Endpoint ID to delete'),
+            nargs='+',
+            help=_('Endpoint(s) to delete (ID only)'),
         )
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
-        identity_client.endpoints.delete(parsed_args.endpoint)
+
+        result = 0
+        for endpoint in parsed_args.endpoints:
+            try:
+                identity_client.endpoints.delete(endpoint)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete endpoint with "
+                          "ID '%(endpoint)s': %(e)s"),
+                          {'endpoint': endpoint, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.endpoints)
+            msg = (_("%(result)s of %(total)s endpoints failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListEndpoint(command.Lister):
-    """List endpoints"""
+    _description = _("List endpoints")
 
     def get_parser(self, prog_name):
         parser = super(ListEndpoint, self).get_parser(prog_name)
@@ -124,7 +146,7 @@ class ListEndpoint(command.Lister):
 
 
 class ShowEndpoint(command.ShowOne):
-    """Display endpoint details"""
+    _description = _("Display endpoint details")
 
     def get_parser(self, prog_name):
         parser = super(ShowEndpoint, self).get_parser(prog_name)
