@@ -20,12 +20,13 @@ import logging
 from keystoneauth1 import exceptions as ks_exc
 from osc_lib.cli import parseractions
 from osc_lib.command import command
+from osc_lib import exceptions
 from osc_lib import utils
 import six
 
 from openstackclient.i18n import _
 from openstackclient.identity import common
-
+from openstackclient.identity.v3 import tag
 
 LOG = logging.getLogger(__name__)
 
@@ -78,6 +79,7 @@ class CreateProject(command.ShowOne):
             action='store_true',
             help=_('Return existing project'),
         )
+        tag.add_tag_option_to_parser_for_create(parser, _('project'))
         return parser
 
     def take_action(self, parsed_args):
@@ -101,6 +103,7 @@ class CreateProject(command.ShowOne):
         kwargs = {}
         if parsed_args.property:
             kwargs = parsed_args.property.copy()
+        kwargs['tags'] = list(set(parsed_args.tags))
 
         try:
             project = identity_client.projects.create(
@@ -148,15 +151,28 @@ class DeleteProject(command.Command):
         domain = None
         if parsed_args.domain:
             domain = common.find_domain(identity_client, parsed_args.domain)
+        errors = 0
         for project in parsed_args.projects:
-            if domain is not None:
-                project_obj = utils.find_resource(identity_client.projects,
-                                                  project,
-                                                  domain_id=domain.id)
-            else:
-                project_obj = utils.find_resource(identity_client.projects,
-                                                  project)
-            identity_client.projects.delete(project_obj.id)
+            try:
+                if domain is not None:
+                    project_obj = utils.find_resource(identity_client.projects,
+                                                      project,
+                                                      domain_id=domain.id)
+                else:
+                    project_obj = utils.find_resource(identity_client.projects,
+                                                      project)
+                identity_client.projects.delete(project_obj.id)
+            except Exception as e:
+                errors += 1
+                LOG.error(_("Failed to delete project with "
+                          "name or ID '%(project)s': %(e)s"),
+                          {'project': project, 'e': e})
+
+        if errors > 0:
+            total = len(parsed_args.projects)
+            msg = (_("%(errors)s of %(total)s projects failed "
+                   "to delete.") % {'errors': errors, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListProject(command.Lister):
@@ -174,12 +190,26 @@ class ListProject(command.Lister):
             metavar='<user>',
             help=_('Filter projects by <user> (name or ID)'),
         )
+        parser.add_argument(
+            '--my-projects',
+            action='store_true',
+            help=_('List projects for the authenticated user. '
+                   'Supersedes other filters.'),
+        )
         parser.add_argument(
             '--long',
             action='store_true',
             default=False,
             help=_('List additional fields in output'),
         )
+        parser.add_argument(
+            '--sort',
+            metavar='<key>[:<direction>]',
+            help=_('Sort output by selected keys and directions (asc or desc) '
+                   '(default: asc), repeat this option to specify multiple '
+                   'keys and directions.'),
+        )
+        tag.add_tag_filtering_option_to_parser(parser, _('projects'))
         return parser
 
     def take_action(self, parsed_args):
@@ -207,7 +237,27 @@ class ListProject(command.Lister):
 
             kwargs['user'] = user_id
 
-        data = identity_client.projects.list(**kwargs)
+        tag.get_tag_filtering_args(parsed_args, kwargs)
+
+        if parsed_args.my_projects:
+            # NOTE(adriant): my-projects supersedes all the other filters.
+            kwargs = {'user': self.app.client_manager.auth_ref.user_id}
+
+        try:
+            data = identity_client.projects.list(**kwargs)
+        except ks_exc.Forbidden:
+            # NOTE(adriant): if no filters, assume a forbidden is non-admin
+            # wanting their own project list.
+            if not kwargs:
+                user = self.app.client_manager.auth_ref.user_id
+                data = identity_client.projects.list(
+                    user=user)
+            else:
+                raise
+
+        if parsed_args.sort:
+            data = utils.sort_items(data, parsed_args.sort)
+
         return (columns,
                 (utils.get_item_properties(
                     s, columns,
@@ -258,6 +308,7 @@ class SetProject(command.Command):
             help=_('Set a property on <project> '
                    '(repeat option to set multiple properties)'),
         )
+        tag.add_tag_option_to_parser_for_set(parser, _('project'))
         return parser
 
     def take_action(self, parsed_args):
@@ -278,6 +329,7 @@ class SetProject(command.Command):
             kwargs['enabled'] = False
         if parsed_args.property:
             kwargs.update(parsed_args.property)
+        tag.update_tags_in_args(parsed_args, project, kwargs)
 
         identity_client.projects.update(project.id, **kwargs)
 
@@ -315,29 +367,28 @@ class ShowProject(command.ShowOne):
         identity_client = self.app.client_manager.identity
 
         project_str = common._get_token_resource(identity_client, 'project',
-                                                 parsed_args.project)
+                                                 parsed_args.project,
+                                                 parsed_args.domain)
 
         if parsed_args.domain:
             domain = common.find_domain(identity_client, parsed_args.domain)
             project = utils.find_resource(
                 identity_client.projects,
                 project_str,
-                domain_id=domain.id,
-                parents_as_list=parsed_args.parents,
-                subtree_as_list=parsed_args.children)
+                domain_id=domain.id)
         else:
             project = utils.find_resource(
                 identity_client.projects,
-                project_str,
-                parents_as_list=parsed_args.parents,
-                subtree_as_list=parsed_args.children)
+                project_str)
 
-        if project._info.get('parents'):
-            project._info['parents'] = [str(p['project']['id'])
-                                        for p in project._info['parents']]
-        if project._info.get('subtree'):
-            project._info['subtree'] = [str(p['project']['id'])
-                                        for p in project._info['subtree']]
+        if parsed_args.parents or parsed_args.children:
+            # NOTE(RuiChen): utils.find_resource() can't pass kwargs,
+            #                if id query hit the result at first, so call
+            #                identity manager.get() with kwargs directly.
+            project = identity_client.projects.get(
+                project.id,
+                parents_as_ids=parsed_args.parents,
+                subtree_as_ids=parsed_args.children)
 
         project._info.pop('links')
         return zip(*sorted(six.iteritems(project._info)))
