@@ -24,6 +24,7 @@ from osc_lib import utils
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 from openstackclient.network import sdk_utils
+from openstackclient.network.v2 import _tag
 
 
 LOG = logging.getLogger(__name__)
@@ -55,6 +56,7 @@ _formatters = {
     'dns_nameservers': utils.format_list,
     'host_routes': _format_host_routes,
     'service_types': utils.format_list,
+    'tags': utils.format_list,
 }
 
 
@@ -132,7 +134,13 @@ def _get_columns(item):
         'subnet_pool_id': 'subnetpool_id',
         'tenant_id': 'project_id',
     }
-    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
+    # Do not show this column when displaying a subnet
+    invisible_columns = ['use_default_subnet_pool']
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(
+        item,
+        column_map,
+        invisible_columns=invisible_columns
+    )
 
 
 def convert_entries_to_nexthop(entries):
@@ -159,6 +167,7 @@ def convert_entries_to_gateway(entries):
 
 def _get_attrs(client_manager, parsed_args, is_create=True):
     attrs = {}
+    client = client_manager.network
     if 'name' in parsed_args and parsed_args.name is not None:
         attrs['name'] = str(parsed_args.name)
 
@@ -171,15 +180,16 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
                 parsed_args.project_domain,
             ).id
             attrs['tenant_id'] = project_id
-        client = client_manager.network
         attrs['network_id'] = client.find_network(parsed_args.network,
                                                   ignore_missing=False).id
         if parsed_args.subnet_pool is not None:
             subnet_pool = client.find_subnet_pool(parsed_args.subnet_pool,
                                                   ignore_missing=False)
             attrs['subnetpool_id'] = subnet_pool.id
+        if parsed_args.use_prefix_delegation:
+            attrs['subnetpool_id'] = "prefix_delegation"
         if parsed_args.use_default_subnet_pool:
-            attrs['use_default_subnetpool'] = True
+            attrs['use_default_subnet_pool'] = True
         if parsed_args.prefix_length is not None:
             attrs['prefixlen'] = parsed_args.prefix_length
         if parsed_args.subnet_range is not None:
@@ -190,10 +200,10 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
             attrs['ipv6_ra_mode'] = parsed_args.ipv6_ra_mode
         if parsed_args.ipv6_address_mode is not None:
             attrs['ipv6_address_mode'] = parsed_args.ipv6_address_mode
-        if parsed_args.network_segment is not None:
-            attrs['segment_id'] = client.find_segment(
-                parsed_args.network_segment, ignore_missing=False).id
 
+    if parsed_args.network_segment is not None:
+        attrs['segment_id'] = client.find_segment(
+            parsed_args.network_segment, ignore_missing=False).id
     if 'gateway' in parsed_args and parsed_args.gateway is not None:
         gateway = parsed_args.gateway.lower()
 
@@ -237,6 +247,7 @@ class CreateSubnet(command.ShowOne):
         parser = super(CreateSubnet, self).get_parser(prog_name)
         parser.add_argument(
             'name',
+            metavar='<name>',
             help=_("New subnet name")
         )
         parser.add_argument(
@@ -252,6 +263,11 @@ class CreateSubnet(command.ShowOne):
             help=_("Subnet pool from which this subnet will obtain a CIDR "
                    "(Name or ID)")
         )
+        subnet_pool_group.add_argument(
+            '--use-prefix-delegation',
+            help=_("Use 'prefix-delegation' if IP is IPv6 format "
+                   "and IP would be delegated externally")
+        )
         subnet_pool_group.add_argument(
             '--use-default-subnet-pool',
             action='store_true',
@@ -330,12 +346,15 @@ class CreateSubnet(command.ShowOne):
             help=_("Set subnet description")
         )
         _get_common_parse_arguments(parser)
+        _tag.add_tag_option_to_parser_for_create(parser, _('subnet'))
         return parser
 
     def take_action(self, parsed_args):
         client = self.app.client_manager.network
         attrs = _get_attrs(self.app.client_manager, parsed_args)
         obj = client.create_subnet(**attrs)
+        # tags cannot be set when created, so tags need to be set later.
+        _tag.update_tags_for_set(client, obj, parsed_args)
         display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
         return (display_columns, data)
@@ -365,8 +384,8 @@ class DeleteSubnet(command.Command):
             except Exception as e:
                 result += 1
                 LOG.error(_("Failed to delete subnet with "
-                          "name or ID '%(subnet)s': %(e)s")
-                          % {'subnet': subnet, 'e': e})
+                          "name or ID '%(subnet)s': %(e)s"),
+                          {'subnet': subnet, 'e': e})
 
         if result > 0:
             total = len(parsed_args.subnet)
@@ -394,7 +413,7 @@ class ListSubnet(command.Lister):
             choices=[4, 6],
             metavar='<ip-version>',
             dest='ip_version',
-            help=_("List only subnets of given IP version in output."
+            help=_("List only subnets of given IP version in output. "
                    "Allowed values for IP version are 4 and 6."),
         )
         dhcp_enable_group = parser.add_mutually_exclusive_group()
@@ -422,14 +441,14 @@ class ListSubnet(command.Lister):
             '--project',
             metavar='<project>',
             help=_("List only subnets which belong to a given project "
-                   "(name or ID) in output")
+                   "in output (name or ID)")
         )
         identity_common.add_project_domain_option_to_parser(parser)
         parser.add_argument(
             '--network',
             metavar='<network>',
             help=_("List only subnets which belong to a given network "
-                   "(name or ID) in output")
+                   "in output (name or ID)")
         )
         parser.add_argument(
             '--gateway',
@@ -448,6 +467,7 @@ class ListSubnet(command.Lister):
                    "(in CIDR notation) in output "
                    "e.g.: --subnet-range 10.10.0.0/16")
         )
+        _tag.add_tag_filtering_option_to_parser(parser, _('subnets'))
         return parser
 
     def take_action(self, parsed_args):
@@ -482,6 +502,7 @@ class ListSubnet(command.Lister):
             filters['name'] = parsed_args.name
         if parsed_args.subnet_range:
             filters['cidr'] = parsed_args.subnet_range
+        _tag.get_tag_filtering_args(parsed_args, filters)
         data = network_client.subnets(**filters)
 
         headers = ('ID', 'Name', 'Network', 'Subnet')
@@ -489,10 +510,10 @@ class ListSubnet(command.Lister):
         if parsed_args.long:
             headers += ('Project', 'DHCP', 'Name Servers',
                         'Allocation Pools', 'Host Routes', 'IP Version',
-                        'Gateway', 'Service Types')
+                        'Gateway', 'Service Types', 'Tags')
             columns += ('project_id', 'is_dhcp_enabled', 'dns_nameservers',
                         'allocation_pools', 'host_routes', 'ip_version',
-                        'gateway_ip', 'service_types')
+                        'gateway_ip', 'service_types', 'tags')
 
         return (headers,
                 (utils.get_item_properties(
@@ -538,11 +559,20 @@ class SetSubnet(command.Command):
                    "'none': This subnet will not use a gateway, "
                    "e.g.: --gateway 192.168.9.1, --gateway none.")
         )
+        parser.add_argument(
+            '--network-segment',
+            metavar='<network-segment>',
+            help=_("Network segment to associate with this subnet (name or "
+                   "ID). It is only allowed to set the segment if the current "
+                   "value is `None`, the network must also have only one "
+                   "segment and only one subnet can exist on the network.")
+        )
         parser.add_argument(
             '--description',
             metavar='<description>',
             help=_("Set subnet description")
         )
+        _tag.add_tag_option_to_parser_for_set(parser, _('subnet'))
         _get_common_parse_arguments(parser, is_create=False)
         return parser
 
@@ -560,7 +590,7 @@ class SetSubnet(command.Command):
             if not parsed_args.no_host_route:
                 attrs['host_routes'] += obj.host_routes
         elif parsed_args.no_host_route:
-            attrs['host_routes'] = ''
+            attrs['host_routes'] = []
         if 'allocation_pools' in attrs:
             if not parsed_args.no_allocation_pool:
                 attrs['allocation_pools'] += obj.allocation_pools
@@ -568,7 +598,10 @@ class SetSubnet(command.Command):
             attrs['allocation_pools'] = []
         if 'service_types' in attrs:
             attrs['service_types'] += obj.service_types
-        client.update_subnet(obj, **attrs)
+        if attrs:
+            client.update_subnet(obj, **attrs)
+        # tags is a subresource and it needs to be updated separately.
+        _tag.update_tags_for_set(client, obj, parsed_args)
         return
 
 
@@ -637,6 +670,7 @@ class UnsetSubnet(command.Command):
                    'Must be a valid device owner value for a network port '
                    '(repeat option to unset multiple service types)')
         )
+        _tag.add_tag_option_to_parser_for_unset(parser, _('subnet'))
         parser.add_argument(
             'subnet',
             metavar="<subnet>",
@@ -672,3 +706,6 @@ class UnsetSubnet(command.Command):
             attrs['service_types'] = tmp_obj.service_types
         if attrs:
             client.update_subnet(obj, **attrs)
+
+        # tags is a subresource and it needs to be updated separately.
+        _tag.update_tags_for_unset(client, obj, parsed_args)
