@@ -335,6 +335,7 @@ class FakeNetwork(object):
             'name': 'network-name-' + uuid.uuid4().hex,
             'status': 'ACTIVE',
             'description': 'network-description-' + uuid.uuid4().hex,
+            'dns_domain': 'example.org.',
             'mtu': '1350',
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
             'admin_state_up': True,
@@ -1132,6 +1133,7 @@ class FakeSecurityGroup(object):
             'description': 'security-group-description-' + uuid.uuid4().hex,
             'project_id': 'project-id-' + uuid.uuid4().hex,
             'security_group_rules': [],
+            'tags': []
         }
 
         # Overwrite default attributes.
@@ -1380,6 +1382,7 @@ class FakeFloatingIP(object):
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
             'description': 'floating-ip-description-' + uuid.uuid4().hex,
             'qos_policy_id': 'qos-policy-id-' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
