@@ -14,6 +14,7 @@
 
 """Volume V2 Volume action implementations"""
 
+import argparse
 import copy
 import logging
 
@@ -37,7 +38,7 @@ def _check_size_arg(args):
     volume is not specified.
     """
 
-    if ((args.snapshot or args.source or args.source_replicated)
+    if ((args.snapshot or args.source)
             is None and args.size is None):
         msg = _("--size is a required option if snapshot "
                 "or source volume is not specified.")
@@ -59,7 +60,7 @@ class CreateVolume(command.ShowOne):
             metavar="<size>",
             type=int,
             help=_("Volume size in GB (Required unless --snapshot or "
-                   "--source or --source-replicated is specified)"),
+                   "--source is specified)"),
         )
         parser.add_argument(
             "--type",
@@ -85,7 +86,7 @@ class CreateVolume(command.ShowOne):
         source_group.add_argument(
             "--source-replicated",
             metavar="<replicated-volume>",
-            help=_("Replicated volume to clone (name or ID)"),
+            help=argparse.SUPPRESS,
         )
         parser.add_argument(
             "--description",
@@ -95,12 +96,12 @@ class CreateVolume(command.ShowOne):
         parser.add_argument(
             '--user',
             metavar='<user>',
-            help=_('Specify an alternate user (name or ID)'),
+            help=argparse.SUPPRESS,
         )
         parser.add_argument(
             '--project',
             metavar='<project>',
-            help=_('Specify an alternate project (name or ID)'),
+            help=argparse.SUPPRESS,
         )
         parser.add_argument(
             "--availability-zone",
@@ -158,7 +159,6 @@ class CreateVolume(command.ShowOne):
 
     def take_action(self, parsed_args):
         _check_size_arg(parsed_args)
-        identity_client = self.app.client_manager.identity
         volume_client = self.app.client_manager.volume
         image_client = self.app.client_manager.image
 
@@ -168,12 +168,6 @@ class CreateVolume(command.ShowOne):
                 volume_client.volumes,
                 parsed_args.source).id
 
-        replicated_source_volume = None
-        if parsed_args.source_replicated:
-            replicated_source_volume = utils.find_resource(
-                volume_client.volumes,
-                parsed_args.source_replicated).id
-
         consistency_group = None
         if parsed_args.consistency_group:
             consistency_group = utils.find_resource(
@@ -186,39 +180,53 @@ class CreateVolume(command.ShowOne):
                 image_client.images,
                 parsed_args.image).id
 
+        size = parsed_args.size
+
         snapshot = None
         if parsed_args.snapshot:
-            snapshot = utils.find_resource(
+            snapshot_obj = utils.find_resource(
                 volume_client.volume_snapshots,
-                parsed_args.snapshot).id
+                parsed_args.snapshot)
+            snapshot = snapshot_obj.id
+            # Cinder requires a value for size when creating a volume
+            # even if creating from a snapshot. Cinder will create the
+            # volume with at least the same size as the snapshot anyway,
+            # so since we have the object here, just override the size
+            # value if it's either not given or is smaller than the
+            # snapshot size.
+            size = max(size or 0, snapshot_obj.size)
 
-        project = None
+        # NOTE(abishop): Cinder's volumes.create() has 'project_id' and
+        # 'user_id' args, but they're not wired up to anything. The only way
+        # to specify an alternate project or user for the volume is to use
+        # the identity overrides (e.g. "--os-project-id").
+        #
+        # Now, if the project or user arg is specified then the command is
+        # rejected. Otherwise, Cinder would actually create a volume, but
+        # without the specified property.
         if parsed_args.project:
-            project = utils.find_resource(
-                identity_client.projects,
-                parsed_args.project).id
-
-        user = None
+            raise exceptions.CommandError(
+                _("ERROR: --project is deprecated, please use"
+                  " --os-project-name or --os-project-id instead."))
         if parsed_args.user:
-            user = utils.find_resource(
-                identity_client.users,
-                parsed_args.user).id
+            raise exceptions.CommandError(
+                _("ERROR: --user is deprecated, please use"
+                  " --os-username instead."))
+        if parsed_args.multi_attach:
+            LOG.warning(_("'--multi-attach' option is no longer supported by "
+                          "the block storage service."))
 
         volume = volume_client.volumes.create(
-            size=parsed_args.size,
+            size=size,
             snapshot_id=snapshot,
             name=parsed_args.name,
             description=parsed_args.description,
             volume_type=parsed_args.type,
-            user_id=user,
-            project_id=project,
             availability_zone=parsed_args.availability_zone,
             metadata=parsed_args.property,
             imageRef=image,
             source_volid=source_volume,
             consistencygroup_id=consistency_group,
-            source_replica=replicated_source_volume,
-            multiattach=parsed_args.multi_attach,
             scheduler_hints=parsed_args.hint,
         )
 
@@ -387,7 +395,6 @@ class ListVolume(command.Lister):
                 'Metadata',
             ]
             column_headers = copy.deepcopy(columns)
-            column_headers[1] = 'Display Name'
             column_headers[4] = 'Type'
             column_headers[6] = 'Attached to'
             column_headers[7] = 'Properties'
@@ -400,7 +407,6 @@ class ListVolume(command.Lister):
                 'Attachments',
             ]
             column_headers = copy.deepcopy(columns)
-            column_headers[1] = 'Display Name'
             column_headers[4] = 'Attached to'
 
         # Cache the server list
@@ -432,7 +438,7 @@ class ListVolume(command.Lister):
             'all_tenants': all_projects,
             'project_id': project_id,
             'user_id': user_id,
-            'display_name': parsed_args.name,
+            'name': parsed_args.name,
             'status': parsed_args.status,
         }
 
@@ -441,6 +447,8 @@ class ListVolume(command.Lister):
             marker=parsed_args.marker,
             limit=parsed_args.limit,
         )
+        column_headers = utils.backward_compat_col_lister(
+            column_headers, parsed_args.columns, {'Display Name': 'Name'})
 
         return (column_headers,
                 (utils.get_item_properties(
